#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 27 11:54:36 2021

@author: adonay
"""

import PySimpleGUIQt as sg
from . import gui_generator as gg


tip = "coma separed patterns, e.g. string1,string2"

def layout_console():
    MLINE_KEY = '-MLINE-__WRITE ONLY__'
    layout = [[sg.Text('Real Time Discord Alert Trader', size=(50,1))],
              [sg.Multiline(size=(1200,None), key=MLINE_KEY, autoscroll=True, enable_events=False),sg.Stretch()]]
    return layout, MLINE_KEY


def layout_portfolio(data_n_headers, font_body, font_header):
    if data_n_headers[0] == []: 
        values = [""*21 ]
    else:
        values=data_n_headers[0]
    
    layout = [
         [sg.Column([[
            sg.Text('Include:  Authors: ', auto_size_text=True,tooltip=tip), sg.Input(key=f'port_filt_author',tooltip=tip),
            sg.Text('Date from: '), sg.Input(key=f'port_filt_date_frm', size=(16, 1), default_text='05/10/2023'),
            sg.Text(' To: '), sg.Input(key=f'port_filt_date_to', size=(16, 1)),
            sg.Text(' Symbols: ', tooltip=tip), sg.Input(key=f'port_filt_sym', tooltip=tip),
            sg.Text(' Channels: ',tooltip=tip), sg.Input(key=f'port_filt_chn',tooltip=tip)
            ],                                        
            [sg.Text("Exclude: |"),
            sg.Checkbox("Closed", key="-port-Closed", enable_events=True),
            sg.Checkbox("Open", key="-port-Open", enable_events=True),
            sg.Checkbox("Cancelled", key="-port-Cancelled", default=True, enable_events=True),
            sg.Checkbox("Neg PnL", key="-port-NegPnL", enable_events=True),
            sg.Checkbox("Pos PnL", key="-port-PosPnL", enable_events=True),
            sg.Checkbox("Live PnL", key="-port-live PnL", enable_events=True),
            sg.Checkbox("Stocks", key="-port-stocks", default=True, enable_events=True),
            sg.Checkbox("Options", key="-port-options", enable_events=True),
            sg.Text('| Authors: ', auto_size_text=True,tooltip=tip), sg.Input(key=f'port_exc_author', tooltip=tip),
            sg.Text('Channels: ', auto_size_text=True,tooltip=tip), sg.Input(key=f'port_exc_chn',tooltip=tip),
            ],
            [sg.ReadButton("Update", button_color=('white', 'black'), key="_upd-portfolio_")]])],
         [sg.Column([[sg.Table(values=values,
                        headings=data_n_headers[1],
                        display_row_numbers=True,
                        auto_size_columns=True,
                        header_font=font_header,
                        text_color='black',
                        font=font_body,
                        justification='left',
                        alternating_row_color='grey',
                        # num_rows=30, #len(data_n_headers[0]),
                        enable_events=True,
                        key='_portfolio_'), sg.Stretch()]])]
         ]
    return layout


def layout_traders(data_n_headers, font_body, font_header):
    
    if data_n_headers[0] == []: 
        values = [""*21 ]
    else:
        values=data_n_headers[0]
    
    layout = [[
        sg.Column([
            [
            sg.Text('Include:  Authors: ', auto_size_text=True,tooltip=tip), sg.Input(key=f'track_filt_author',tooltip=tip),
            sg.Text('Date from: '), sg.Input(key=f'track_filt_date_frm', default_text='05/10/2023', size=(16, 1)),
            sg.Text(' To: '), sg.Input(key=f'track_filt_date_to', size=(16, 1)),
            sg.Text(' Symbols: ',tooltip=tip), sg.Input(key=f'track_filt_sym',tooltip=tip),
            sg.Text(' Channels: ',tooltip=tip), sg.Input(key=f'track_filt_chn',tooltip=tip)
            ],[ 
            sg.Text("Exclude: |"),
            sg.Checkbox("Closed", key="-track-Closed", enable_events=True),
            sg.Checkbox("Open", key="-track-Open", enable_events=True),
            sg.Checkbox("Neg PnL", key="-track-NegPnL", enable_events=True),
            sg.Checkbox("Pos PnL", key="-track-PosPnL", enable_events=True),
            sg.Checkbox("Live PnL", key="-track-live PnL", enable_events=True), 
            sg.Checkbox("Stocks", key="-track-stocks", default=True, enable_events=True),
            sg.Checkbox("Options", key="-track-options", enable_events=True),
            sg.Text('| Authors: ', auto_size_text=True,tooltip=tip), sg.Input(key=f'track_exc_author', tooltip=tip),
            sg.Text('Channels: ', auto_size_text=True,tooltip=tip), sg.Input(key=f'track_exc_chn',tooltip=tip),
            ],[sg.ReadButton("Update", button_color=('white', 'black'), key="_upd-track_")]
            ])],
         [sg.Column([
            [
            sg.Table(values=values,
                headings=data_n_headers[1],
                display_row_numbers=True,
                auto_size_columns=True,
                header_font=font_header,
                text_color='black',
                font=font_body,
                justification='left',
                alternating_row_color='grey',
                # num_rows=30, #len(data_n_headers[0]),
                key='_track_'), sg.Stretch()]])]
         ]
    return layout


def layout_stats(data_n_headers, font_body, font_header):
    
    if data_n_headers[0] == []: 
        values = [""*21 ]
    else:
        values=data_n_headers[0]
    
    layout = [
        [sg.Column([[sg.Text('Include:  Authors: ', auto_size_text=True, tooltip=tip), sg.Input(key=f'stat_filt_author', tooltip=tip),
                     sg.Text('Date from:'), sg.Input(key=f'stat_filt_date_frm', size=(16, 1), default_text='05/10/2023'),
                     sg.Text(' To:', size=(5, 1)), sg.Input(key=f'stat_filt_date_to', size=(16, 1)),
                     sg.Text(' Symbols:'), sg.Input(key=f'stat_filt_sym', tooltip=tip),
                     sg.Text(' Max $:', tooltip="calculate stats limiting trades to max $"), 
                     sg.Input(key=f'stat_max_trade_cap', tooltip="calculate stats limiting trades to max $"),
                     sg.Text(' Max quantity:', tooltip="calculate stats limiting trades to max quantity"), 
                     sg.Input(key=f'stat_max_qty', tooltip="calculate stats limiting trades to max quantity"),
                     ],
                     [sg.Text("Exclude: "),
                      sg.Checkbox("Neg PnL", key="-stat-NegPnL", enable_events=True),
                      sg.Checkbox("Pos PnL", key="-stat-PosPnL", enable_events=True),                  
                      sg.Checkbox("Stocks", key="-stat-stocks", default=True, enable_events=True),
                      sg.Checkbox("Options", key="-stat-options", enable_events=True),
                      sg.Text('| Authors: ', auto_size_text=True,tooltip=tip), sg.Input(key=f'stat_exc_author', tooltip=tip),
                      sg.Text('Symbols: ', auto_size_text=True,tooltip=tip), sg.Input(key=f'stat_exc_sym',tooltip=tip),
                      sg.Text('Channels: ', auto_size_text=True,tooltip=tip), sg.Input(key=f'stat_exc_chn',tooltip=tip),
                      ],
                     [sg.ReadButton("Update", button_color=('white', 'black'), key="_upd-stat_")]])
                    ],
         [sg.Column([[sg.Table(values=values,
                        headings=data_n_headers[1],
                        display_row_numbers=True,
                        auto_size_columns=True,
                        header_font=font_header,
                        text_color='black',
                        font=font_body,
                        justification='left',
                        alternating_row_color='grey',
                        # num_rows=30, #len(data_n_headers[0]),
                        key='_stat_'), sg.Stretch()]])]
         ]
    return layout

def layout_chan_msg(chn, data_n_headers, font_body, font_header):    
    # Handle empy chan history
    if data_n_headers[0] == []: 
        values = [[""*len(data_n_headers[1])] ]
    else:
        values=data_n_headers[0]

    layout = [
        [sg.Text('Filter:  Authors: '), sg.Input(key=f'{chn}_filt_author'),
           # sg.Text(' '*2),
         sg.Text('Date from: '), sg.Input(key=f'{chn}_filt_date_frm', default_text='05/09/23'),
         sg.Text(' To: '), sg.Input(key=f'{chn}_filt_date_to'),
          # sg.Text(' '*1),
         sg.Text('Message contains: '), sg.Input(key=f'{chn}_filt_cont'),
         sg.Text('Num. rows display: '), sg.Input(key=f'{chn}_n_rows'),
         ],
        [sg.ReadFormButton("Update", button_color=('white', 'black'), key=f'{chn}_UPD', bind_return_key=True)],
        [sg.Column([[sg.Table(values=values,
                  headings=data_n_headers[1],
                  justification='left',
                  display_row_numbers=False,
                  text_color='black',
                  font=font_body,
                  # col_widths=[30,200, 300],
                  header_font=font_header,
                  auto_size_columns =True, max_col_width=30,
                  # auto_size_columns=True,
                  # vertical_scroll_only=False,
                   alternating_row_color='grey',
                  # col_widths=[30,300, 1300],
                  # row_height=20,
                  # num_rows=30,
                  # enable_events = False,
                  # bind_return_key = True,
                #   tooltip = "Selecting row and pressing enter will parse message",
                  key=f"{chn}_table")]])]
        ]
    return layout


def tt_acnt(text, fsize=12, bold=True, underline=True, font_name="Arial", size=None, k=None):
    font = [font_name, fsize]
    if bold: font += ['bold']
    if underline: font += ["underline"]
    if size is None:
        size = (len(text) *2, 1)
    if k is not None:
        return sg.T(text,font=font,size=size, key=k)
    else:
        return sg.T(text,font=font,size=size)


def layout_account(bksession, font_body, font_header):
    if bksession is None:
        return [[sg.T("No brokerage API provided in config.ini")]]
    acc_inf, ainf = gg.get_acc_bals(bksession)
    pos_tab, pos_headings = gg.get_pos(acc_inf)
    ord_tab, ord_headings, _= gg.get_orders(acc_inf)

    layout = [[sg.Column([
        [tt_acnt("Account ID:", font_body[1]), tt_acnt(ainf["id"], font_body[1], 0, 0, font_body[0]),
         tt_acnt("Balance:", font_body[1]), tt_acnt("$" + str(ainf["balance"]), font_body[1], 0, 0, font_body[0], k="acc_b"),
         tt_acnt("Cash:", font_body[1]), tt_acnt("$" + str(ainf["cash"]), font_body[1], 0, 0, font_body[0], k="acc_c"),
         tt_acnt("Funds:", font_body[1]), tt_acnt("$" + str(ainf["funds"]), font_body[1], 0, 0, font_body[0], k="acc_f")
         ],[sg.ReadFormButton("Update", button_color=('white', 'black'), key='acc_updt', bind_return_key=True)]
             ])],
        [sg.Column(
            [
             [sg.T("Positions", font=(font_body[0], font_body[1], 'bold', "underline"),size=(20,1.5))],
             [sg.Table(values=pos_tab, headings=pos_headings,justification='left',
              display_row_numbers=False, text_color='black', font=font_body,
               auto_size_columns=True,
               header_font=font_header,
              alternating_row_color='grey',
               max_col_width=30,
              key='_positions_')]]),
        sg.Column(
            [
             [sg.T("Orders",font=(font_header[0], font_header[1], 'bold', "underline"),size=(20,1.5))],
             [sg.Table(values=ord_tab, headings=ord_headings,justification='left',
              display_row_numbers=False, text_color='black', font=font_body,
              auto_size_columns=True, 
              header_font=font_header,
              key='_orders_')]])
            ]]
    return layout


def update_acct_ly(bksession, window):

    acc_inf, ainf = gg.get_acc_bals(bksession)
    pos_tab, _ = gg.get_pos(acc_inf)
    ord_tab, _, _= gg.get_orders(acc_inf)

    window.Element("acc_b").update(ainf["balance"])
    window.Element("acc_c").update(ainf["cash"])
    window.Element("acc_f").update(ainf["funds"])

    window.Element("_positions_").update(pos_tab)
    window.Element("_orders_").update(ord_tab)
    
    for el in ["_positions_", "_orders_"]:
        window.Element(el).Widget.resizeRowsToContents()
        window.Element(el).Widget.resizeColumnsToContents()





import os

import unittest
import pandas as pd
import numpy as np
from DiscordAlertsTrader.gui_generator import get_stats_data
from DiscordAlertsTrader.configurator import cfg
from DiscordAlertsTrader.alerts_tracker import calc_stc_prices

class TestConfigurator(unittest.TestCase):

    def test_get_stats_data(self):
        fname_port="test_tracker_portfolio.csv"
        
        # make dummy portfolio
        cols = cfg["col_names"]["tracker_portfolio"]
        dt = "2023-05-15 10:02:47.409000,NFLX_051923C345,test#3069,chan,0,option,BTO,10,5,12,,,,,,,,,,,,,2023-05-16 12:39:37.884000,".split(',')
        port = pd.DataFrame([dt], columns=cols.split(","))
        port = port.replace('', np.nan)
        order = {
            "price":12,
            "Actual Cost":24,
            'uQty' :1,
            }
        stc_info = calc_stc_prices(port.loc[0], order)
        for k, v in stc_info.items():
            port.loc[0,k] = v
        port.to_csv(fname_port, index=False)
        
        data, _ = get_stats_data(exclude={}, stat_filt_author='', stat_filt_date_frm='',
                    stat_filt_date_to='', stat_filt_sym='', 
                    stat_max_trade_cap='', stat_max_qty='', trail_stop_perc='',
                    fname_port=fname_port)
        self.assertTrue(data[0][1:] == data[1][1:])
    
        data, h = get_stats_data(exclude={}, stat_filt_author='', stat_filt_date_frm='',
                    stat_filt_date_to='', stat_filt_sym='', 
                    stat_max_trade_cap='', stat_max_qty='1', trail_stop_perc='',
                    fname_port=fname_port)
        
        expected = [
            ['Trader', 'test'],
            ['PnL$', '200'],
            ['PnL$-current', '1200'],
            ['PnL', ' 20'],
            ['PnL-current', '100'],
            ['PnL diff', ' 80'],
            ['N Trades', '  1'],
            ['Since', '05/15/2023'],
            ['Last', '05/15/2023']
            ]

        for k,v, exp in zip(h,data[0], expected):
            self.assertTrue(k == exp[0])
            self.assertTrue(v == exp[1])

        # Delete the generated file
        os.remove(fname_port)

if __name__ == '__main__':
    unittest.main()



import configparser
import os
import os.path as op
import json

package_dir = os.path.abspath(os.path.dirname(__file__))

config_path = package_dir + '/config.ini'
if not os.path.exists(config_path):
    print("\033[91mWARNING: DiscordAlertsTrader/config.ini not found. \033[0m")
    print("\033[91mWARNING: Rename DiscordAlertsTrader/config_example.ini to DiscordAlertsTrader/config.ini. \033[0m")
    print("\033[91mWARNING: Reverting to config_example.ini for now (might be necessary for testing). \033[0m")
    config_path = package_dir + '/config_example.ini'

# load configuration file
cfg = configparser.ConfigParser()
cfg.read(config_path, encoding='utf-8')

# change data_dir if it is just a folder name
data_dir = cfg['general']['data_dir']
_, ext = os.path.splitext(data_dir)
if ext == "":    
    cfg['general']['data_dir'] = os.path.join(package_dir, "..", data_dir)
    print("full data dir:", cfg['general']['data_dir'])

# add path to file names
for k, v in cfg['portfolio_names'].items():
    cfg['portfolio_names'][k] = op.join(cfg['general']['data_dir'], v)
cfg['portfolio_names']['mock_portfolio_fname'] = './tests/trader_portfolio_simulated.csv'
cfg['portfolio_names']['mock_alerts_log_fname'] = './tests/trader_logger_simulated.csv'

# Define column names for portfolios and hist messages
portfolio_cols = ",".join([
                "Date", "Symbol", "Trader", "isOpen", "BTO-Status", "Asset", "Type", "Price", "Price-Alert", "Price-Current",
                "uQty", "filledQty", "Avged", "Avged-prices", "exit_plan", "ordID", "Risk", "SL_mental","PnL", "$PnL",
                "PnL-Alert", "$PnL-Alert","PnL-Current","$PnL-Current"
                ] + [
                    "STC%d-%s"% (i, v) for v in
                    ["Alerted", "Status", "xQty", "uQty", "Price", "Price-Alerted", "Price-Current", "PnL","Date", "ordID"]
                    for i in range(1,4)])

tracker_portfolio_cols = ",".join([
                "Date", "Symbol", "Trader", 'Channel', "isOpen", "Asset", "Type", "Price", "Amount", "Price-current", "Prices", "Prices-current", "Avged"
                ] + [ f"STC-{v}" for v in
                    ["Amount", "Price", "Price-current", "Prices", "Prices-current", "PnL", "PnL-current","PnL$", "PnL$-current", "Date"]
                    for i in range(1,2)] + ["TrailStats"])
cfg["col_names"] = {
    'portfolio': portfolio_cols,
    'alerts_log': 'Date,Symbol,Trader,action,parsed,msg,portfolio_idx',
    "tracker_portfolio": tracker_portfolio_cols,
    "chan_hist": 'AuthorID,Author,Date,Content,Parsed'
    } 

# get chan IDs in a dict format
channel_ids_str = cfg.get('discord', 'channel_IDS')
channel_ids = json.loads(channel_ids_str.replace("\n", "").replace(",}", "}"))


from DiscordAlertsTrader.alerts_tracker import AlertsTracker

tracker = AlertsTracker(brokerage=None,
                 portfolio_fname='tests/data/analysts_portfolio.csv',
                 dir_quotes='tests/data/live_quotes'
                 )


open_trade = tracker.portfolio.index[tracker.portfolio['Symbol'] == 'QQQ_051223C327'].to_list()[-1]
trailstat = tracker.compute_trail(open_trade)
assert(trailstat == 'min,-87.1%,$0.04,in 02:09:36| max,3.23%,$0.32,in 00:01:19| | TS:0.2,-19.35%,$0.25,in 00:08:14| TS:0.3,-29.03%,$0.22,in 00:16:39| TS:0.4,-38.71%,$0.19,in 00:17:09| TS:0.5,-48.39%,$0.16,in 00:22:04')



from abc import ABC, abstractmethod
from ..configurator import cfg

class BaseBroker(ABC):
    @abstractmethod
    def __init__(self, api_key, secret_key, passphrase):
        pass

    @abstractmethod
    def get_session(self):
        pass

    @abstractmethod
    def get_quotes(self, symbol):
        pass

    @abstractmethod
    def send_order(self, side, symbol, order_type, quantity, price=None, stop_price=None):
        pass

    @abstractmethod
    def cancel_order(self, order_id):
        pass

    @abstractmethod
    def get_orders(self):
        pass

    # @abstractmethod
    # def get_order_status(self, order_id):
    #     pass
    
    def get_order_info(self, order_id):
        #     sold_unts = order_info['orderLegCollection'][0]['quantity']

        # if 'price' in order_info.keys():
        #     stc_price = order_info['price']
        # elif 'stopPrice' in order_info.keys():
        #     stc_price = order_info['stopPrice']
        # elif "orderActivityCollection" in order_info.keys():
        #     prics = []
        #     for ind in order_info["orderActivityCollection"]:
        #         prics.append([ind['quantity'], ind['executionLegs'][0]['price']])
        #         n_tot= sum([i[0] for i in prics])
        #     stc_price =  sum([i[0]*i[1] for i in prics])/ n_tot

        # bto_price = self.portfolio.loc[open_trade, "Price"]
        # bto_price_alert = self.portfolio.loc[open_trade, "Price-Alert"]
        # bto_price_current = self.portfolio.loc[open_trade, "Price-Current"]
        # stc_PnL = float((stc_price - bto_price)/bto_price) *100

        # xQty = sold_unts/ self.portfolio.loc[open_trade, "uQty"]

        # date = order_info["closeTime"]
        pass


def get_brokerage(name=cfg['general']['BROKERAGE']):
    if name.lower() == 'tda':
        from .TDA_api import TDA
        tda = TDA()
        tda.get_session()
        return tda
    elif name == "webull":
        NotImplemented
    elif name.lower() == 'etrade':
        from .eTrade_api import eTrade
        et = eTrade()
        try:
            et.get_session()
        except Exception as e:
            print("Got error: \n", e, "\n Trying again...if it fails again, rerun the application.")
            et.get_session()
        return et


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Apr  3 18:18:43 2021

@author: adonay
"""
import os
import os.path as op
import threading
import pandas as pd
from datetime import datetime
import time
import re
import queue
import PySimpleGUIQt as sg
from PySide2.QtWidgets import QHeaderView

from DiscordAlertsTrader.brokerages import get_brokerage
from DiscordAlertsTrader import gui_generator as gg
from DiscordAlertsTrader import gui_layouts as gl
from DiscordAlertsTrader.discord_bot import DiscordBot
from DiscordAlertsTrader.configurator import cfg, channel_ids

def match_authors(author_str:str)->str:
    """Author have an identifier in discord, it will try to find full author name

    Parameters
    ----------
    author_str : str
        string to match the author

    Returns
    -------
    str
        author with identifier
    """
    if "#" in author_str:
        return author_str
    authors = []
    for chn in channel_ids.keys():
        at = pd.read_csv(op.join(cfg['general']['data_dir'] , f"{chn}_message_history.csv"))["Author"].unique()
        authors.extend(at)
    authors = list(dict.fromkeys(authors))
    authors = [a for a in authors if author_str.lower() in a.lower()]
    if len(authors) == 0:
        author = f"{author_str}#No match, find author identifier#1234"
    elif len(authors) > 1:
        author = f"{author_str}#Multiple matches, find author identifier#1234"
    else:
        author = authors[0]
    return author

def fit_table_elms(Widget_element):
    Widget_element.resizeRowsToContents()
    Widget_element.resizeColumnsToContents()
    Widget_element.resizeRowsToContents()
    Widget_element.resizeColumnsToContents()

# sg.theme('Dark Blue 3')
sg.SetOptions(font=("Helvitica 10"))
                # element_padding=(0, 0), margins=(1, 1))

fnt_b = "Arial 10"
fnt_h = "Arial 10"

ly_cons, MLINE_KEY = gl.layout_console()

def mprint(*args, **kwargs):
    window[MLINE_KEY].print(*args, **kwargs)

print(1)
gui_data = {}
gui_data['port'] = gg.get_portf_data()
ly_port = gl.layout_portfolio(gui_data['port'], fnt_b, fnt_h)

gui_data['trades'] = gg.get_tracker_data()
ly_track = gl.layout_traders(gui_data['trades'], fnt_b, fnt_h)

gui_data['stats'] = gg.get_stats_data()
ly_stats = gl.layout_stats(gui_data['stats'], fnt_b, fnt_h)
print(2)

chns = channel_ids.keys()
ly_chns = []
for chn in chns:
    chn_fname = cfg['general']['data_dir']+f"/{chn}_message_history.csv"
    if not op.exists(chn_fname):
        os.makedirs(cfg['general']['data_dir'], exist_ok=True)
        pd.DataFrame(columns=cfg["col_names"]['chan_hist'].split(',')).to_csv(chn_fname, index=False)
    gui_data[chn] = gg.get_hist_msgs(chan_name=chn)
    ly_ch = gl.layout_chan_msg(chn, gui_data[chn], fnt_b, fnt_h)
    ly_chns.append(ly_ch)

bksession = get_brokerage()
ly_accnt = gl.layout_account(bksession, fnt_b, fnt_h)

layout = [[sg.TabGroup([
                        [sg.Tab("Console", ly_cons, font=fnt_b)],
                        [sg.Tab('Portfolio', ly_port)],
                        [sg.Tab('Analysts portfolio', ly_track)],
                        [sg.Tab('Analysts stats', ly_stats)],
                        [sg.Tab(c, h) for c, h in zip(chns, ly_chns)],                        
                        [sg.Tab("Account", ly_accnt)]
                        ], title_color='black')],
          [sg.Input(default_text="Author#1234, STC 1 AAA 115C 05/30 @2.5 [click portfolio row number to prefill]",
                    size= (110,1.5), key="-subm-msg",
                    tooltip="Click portfolio row number to prefill the STC alert"),
           sg.Button("Trigger alert", key="-subm-alert", tooltip="Will generate alert in portfolio and tracker", size= (20,1))]
        ]
print(3)
window = sg.Window('Discord Alerts Trader for BullTrades', layout,size=(100, 800), # force_toplevel=True,
                    auto_size_text=True, resizable=True)
print(4)
def mprint_queue(queue_item_list):
    # queue_item_list = [string, text_color, background_color]
    kwargs = {}
    text = queue_item_list[0]
    len_que = len(queue_item_list)
    if len_que == 2:
        kwargs["text_color"] = queue_item_list[1]
    elif len_que == 3:
        tcol = queue_item_list[1]
        tcol = "black" if tcol == "" else tcol
        kwargs["text_color"] = tcol

        bcol = queue_item_list[2]
        bcol = "white" if bcol == "" else bcol
        kwargs["background_color"] = bcol

    window[MLINE_KEY].print(text, **kwargs)

def update_portfolios_thread(window):
    while True:
        time.sleep(60)
        window["_upd-portfolio_"].click()
        time.sleep(2)  
        window["_upd-track_"].click()
print(5)
event, values = window.read(.1)

els = ['_portfolio_', '_track_', ] + [f"{chn}_table" for chn in chns]
els = els + ['_orders_', '_positions_'] if bksession is not None else els
for el in els:
    try:
        fit_table_elms(window.Element(el).Widget)
    except:
        pass

for chn in chns:
    table = window[f"{chn}_table"].Widget.horizontalHeader()
    table.setSectionResizeMode(2, QHeaderView.Stretch)
    window[f"{chn}_table"].Widget.scrollToBottom()

print(6)
event, values = window.read(.1)
print(7)
trade_events = queue.Queue(maxsize=20)
alistner = DiscordBot(trade_events, brokerage=bksession)
print(8)
threading.Thread(target=update_portfolios_thread, args=(window,), daemon=True).start()
print(9)
event, values = window.read(.1)

# exclusion filters for the portfolio and analysts tabs
port_exc = {"Closed":False,
            "Open":False,
            "NegPnL":False,
            "PosPnL":False,
            "live PnL":False,
            "stocks":True,
            "options":False,
            }
track_exc = port_exc.copy()
stat_exc = port_exc.copy()
port_exc["Cancelled"] = True

print(10)
dt, _  = gg.get_tracker_data(track_exc, **values)
window.Element('_track_').Update(values=dt)
fit_table_elms(window.Element("_track_").Widget)
dt, hdr = gg.get_portf_data(port_exc)
window.Element('_portfolio_').Update(values=dt)
fit_table_elms(window.Element("_portfolio_").Widget)
dt, hdr = gg.get_stats_data(port_exc)
window.Element('_stat_').Update(values=dt)
fit_table_elms(window.Element("_stat_").Widget)

def run_gui():  
    while True:    
        event, values = window.read(1)#.1)

        if event == sg.WINDOW_CLOSED:
            break
        if '_portfolio_' in event and values['_portfolio_'] != []:
            pix = values['_portfolio_'][0]
            dt, hdr = gg.get_portf_data(port_exc, **values)
            symb = dt[pix][hdr.index('Symbol')]
            auth = match_authors(dt[pix][hdr.index('Trader')])
            qty = dt[pix][hdr.index('filledQty')]
            price = dt[pix][hdr.index('1-$-Current')]
            if "_" in symb:
                # option
                exp = r"(\w+)_(\d{6})([CP])([\d.]+)"        
                match = re.search(exp, symb, re.IGNORECASE)
                if match:
                    symbol, date, type, strike = match.groups()
                    symb_str = f"{auth}, STC {qty} {symbol} {strike}{type} {date[:2]}/{date[2:4]} @{price}"
            else:
                symb_str= f"{auth}, STC {qty} {symb} @{price}"
            window.Element("-subm-msg").Update(value=symb_str)
            
        if event == "_upd-portfolio_": # update button in portfolio
            ori_col = window.Element("_upd-portfolio_").ButtonColor
            window.Element("_upd-portfolio_").Update(button_color=("black", "white"))
            event, values = window.read(.1)
            dt, _ = gg.get_portf_data(port_exc, **values)
            window.Element('_portfolio_').Update(values=dt)
            fit_table_elms(window.Element("_portfolio_").Widget)
            window.Element("_upd-portfolio_").Update(button_color=ori_col)

        elif event == "_upd-track_": # update button in analyst alerts
            ori_col = window.Element(f'_upd-track_').ButtonColor
            window.Element("_upd-track_").Update(button_color=("black", "white"))
            event, values = window.read(.1)
            dt, _  = gg.get_tracker_data(track_exc, **values)
            window.Element('_track_').Update(values=dt)
            fit_table_elms(window.Element("_track_").Widget)
            window.Element("_upd-track_").Update(button_color=ori_col)

        elif event == "_upd-stat_": # update button in analyst stats
            ori_col = window.Element(f'_upd-stat_').ButtonColor
            window.Element("_upd-stat_").Update(button_color=("black", "white"))
            event, values = window.read(.1)
            dt, _  = gg.get_stats_data(stat_exc, **values)
            window.Element('_stat_').Update(values=dt)
            fit_table_elms(window.Element("_stat_").Widget)
            window.Element("_upd-stat_").Update(button_color=ori_col)

        elif event.startswith("-port-"): # radial click, update portfolio
            key =  event.replace("-port-", "")
            state = window.Element(event).get()
            port_exc[key] = state
            dt, _ = gg.get_portf_data(port_exc, **values)
            window.Element('_portfolio_').Update(values=dt)

        elif event.startswith("-track-"): # radial click, update analyst alerts
            key =  event.replace("-track-", "")
            state = window.Element(event).get()
            track_exc[key] = state
            dt, _ = gg.get_tracker_data(track_exc, **values)
            window.Element('_track_').Update(values=dt)

        elif event.startswith("-stat-"): # radial click, update analyst stats
            key =  event.replace("-stat-", "")
            state = window.Element(event).get()
            stat_exc[key] = state
            dt, _ = gg.get_stats_data(stat_exc, **values)
            window.Element('_stat_').Update(values=dt)

        elif event[-3:] == "UPD":
            chn = event[:-4]
            ori_col = window.Element(f'{chn}_UPD').ButtonColor
            window.Element(f'{chn}_UPD').Update(button_color=("black", "white"))
            event, values = window.read(.1)

            args = {}
            for k, v in values.items():
                if k[:len(chn)] == chn:
                    args[k[len(chn)+1:]] = v
            dt, _  = gg.get_hist_msgs(chan_name=chn, **args)
            if args['n_rows'] != "":
                n_rows = eval(args['n_rows'])
                n_rows = max(1, n_rows)
                window.Element(f"{chn}_table").Update(values=dt,  num_rows=n_rows)
            else:
                window.Element(f"{chn}_table").Update(values=dt)

            fit_table_elms(window.Element(f"{chn}_table").Widget)
            window.Element(f'{chn}_UPD').Update(button_color=ori_col)

        elif event == 'acc_updt':
            ori_col = window.Element("acc_updt").ButtonColor
            window.Element("acc_updt").Update(button_color=("black", "white"))
            event, values = window.read(.1)
            gl.update_acct_ly(bksession, window)
            fit_table_elms(window.Element(f"_positions_").Widget)
            fit_table_elms(window.Element(f"_orders_").Widget)
            window.Element("acc_updt").Update(button_color=ori_col)

        elif event == "-subm-alert":
            ori_col = window.Element("-subm-alert").ButtonColor
            window.Element("-subm-alert").Update(button_color=("black", "white"))
            event, values = window.read(.1)
            try:        
                author, msg = values['-subm-msg'].split(',')
            except ValueError:
                author, msg = values['-subm-msg'].split(':')
            author = match_authors(author.strip())
            msg = msg.strip()
            date = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")
            new_msg = pd.Series({
                'AuthorID': None,
                'Author': author,
                'Date': date, 
                'Content': msg,
                'Channel': "GUI_input"
                })
            alistner.new_msg_acts(new_msg, from_disc=False)
            window.Element("-subm-alert").Update(button_color=ori_col)

        try:
            event_feedb = trade_events.get(False)
            mprint_queue(event_feedb)
        except queue.Empty:
            pass



def run_client():
    alistner.run(cfg['discord']['discord_token'])


def gui():   
    client_thread = threading.Thread(target=run_client)

    # start the threads
    client_thread.start()
    run_gui()

    # close the GUI window
    window.close()
    alistner.close_bot()
    exit()


if __name__ == '__main__':
    gui()



import os
from setuptools import setup, find_packages
from pkg_resources import parse_requirements


# Read requirements from requirements.txt
with open('requirements.txt') as f:
    requirements = [str(req) for req in parse_requirements(f)]

# get the version
version = None
with open(os.path.join('DiscordAlertsTrader', '__init__.py'), 'r') as fid:
    for line in (line.strip() for line in fid):
        if line.startswith('__version__'):
            version = line.split('=')[1].strip().strip('\'')
            break

setup(
    name='DiscordAlertsTrader',
    version='0.9.1',
    author='Adonay Nunes',
    author_email='adonays.nunes@gmail.com',
    description='Package for automating discord trade alerts in TDA or eTrade.',
    license='BSD (3-clause)',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='',
    download_url='https://github.com/AdoNunes/DiscordAlertsTrader',
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3.9',
        'Topic :: Communications :: Chat',
        'Topic :: Office/Business :: Financial',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
        'Intended Audience :: Financial and Insurance Industry',
        'Intended Audience :: Information Technology',
        'Operating System :: Microsoft :: Windows',
    ],
    entry_points = {'console_scripts': ['DiscordAlertsTrader = DiscordAlertsTrader.gui:gui'],
          }
)


import place_order as pord


Session = pord.get_TDsession()


df_pos, df_ordr = pord.get_positions_orders(Session)


order = {"symbol": "ASO", 
         "uQty":1,
         "price":2,
         'strike':None,
         "price": 0.5,
         "PTs": [1,2,3],
         "PTs_Qty":[1,1,1],
         "SL": 0.1,
         "SL_stop": 0.2
         }
new_order_bto_lim = pord.make_BTO_lim_order(**order)

order_response, order_id =  pord.send_order(new_order_bto_lim, Session)
new_order=  pord.make_Lim_SL_order(**order)
new_order =  pord.make_STC_lim(**order)


resp = Session.get_quotes(instruments=[order['symbol']])
if resp[order['symbol']].get('description' ) == 'Symbol not found':
    print (f"{order['symbol']} not found during price quote")
else:
    quote = resp[order['symbol']]['lastPrice']
    print (f"{order['symbol']} quote: {quote}")





import webbrowser
import pandas as pd
import pyetrade
import re
import random
from datetime import datetime
import time
import json
import os

from ..configurator import cfg
from . import BaseBroker

class eTrade(BaseBroker):
    def __init__(self, account_n=0, accountId=None):
        self.base_url = cfg["etrade"]["PROD_BASE_URL"]
        self.accountId = accountId
        self.account_n = account_n
        self.consumer_key = cfg["etrade"]["CONSUMER_KEY"]
        self.consumer_secret = cfg["etrade"]["CONSUMER_SECRET"]
        
    def get_session(self):
        """get token and sessions, will try several times and sleep for a second between each try"""
        for ix in range(5):
            try:
                return self._get_session()
            except:
                print(ix,"Could not get session, trying again")
                time.sleep(1)
        raise Exception("Could not get session")

    def _get_access_token(self, oauth,verifier_code):
        """Gets access token and tries 3 times before giving up"""        
        for ix in range(3):
            try:
                request_token = oauth.get_access_token(verifier_code)
                return request_token                
            except:
                print(f"Could not get token, trying again {ix}/3")
                time.sleep(1)
        raise Exception("Could not get token")

    def _get_session(self):
        """Allows user authorization for the sample application with OAuth 1"""
        def sessions():
            # get sessions
            kwargs = {
                'client_key': self.consumer_key,
                'client_secret': self.consumer_secret,
                'resource_owner_key': self.tokens['oauth_token'],
                'resource_owner_secret': self.tokens['oauth_token_secret'],
                'dev': False
                }
            self.account_session = pyetrade.ETradeAccounts(**kwargs)
            self.market_session = pyetrade.ETradeMarket(**kwargs)     
            self.order_session = pyetrade.ETradeOrder(**kwargs)
            self._get_account()
            return True
        
        # if tokens saved try getting session
        if os.path.exists("tokens.json"):
            with open("tokens.json", "r") as f:
                self.tokens = json.load(f)   
            try:
                return sessions()  
            except:
                print("Loaded tokens expired, requesting new tokens")
                os.remove("tokens.json")  
        
        # if tokens not valid, get new ones
        oauth = pyetrade.ETradeOAuth(self.consumer_key, self.consumer_secret)
        if cfg['etrade'].getboolean('WITH_BROWSER'):
            webbrowser.open(oauth.get_request_token())
        else:
            print("Please open the following URL in your browser:")
            print(oauth.get_request_token())
        verifier_code = input("Please accept agreement and enter verification code from browser: ")
        self.tokens = self._get_access_token(oauth, verifier_code)
        with open("tokens.json", "w") as f:
            json.dump(self.tokens, f)
        return sessions() 

    def _get_account(self):
        """
        Calls account list API to retrieve a list of the user's E*TRADE accounts
        """
        data = self.account_session.list_accounts(resp_format='json')
        self.accounts_list = data["AccountListResponse"]["Accounts"]["Account"]        
        
        if self.accountId is not None:
            self.accountIdKey = [self.accounts_list[i]['accountIdKey'] for i in range(len(self.accounts_list)) 
                                 if self.accounts_list[i]['accountId'] == self.accountId][0]
        else:
            self.accountIdKey = self.accounts_list[self.account_n]['accountIdKey']
            self.accountId = self.accounts_list[self.account_n]['accountId']
        self.account = self.accounts_list[self.account_n]

    def get_account_info(self):
        """
        Call portfolio API to retrieve a list of positions held in the specified account
        """
        data = self.account_session.get_account_balance(self.accountIdKey, resp_format='json')        
        balance= {
            'liquidationValue': data['BalanceResponse'].get("Computed").get("RealTimeValues").get("totalAccountValue"),
            'cashBalance': data['BalanceResponse'].get("Computed").get('cashBalance'),
            'availableFunds': data['BalanceResponse'].get("Computed").get('cashAvailableForInvestment'),
            }

        data = self.account_session.get_account_portfolio(self.accountIdKey, resp_format='json')
                
        acc_inf ={
            'securitiesAccount':{   
                'positions':[],
                'accountId' : self.accountId,
                'currentBalances':{
                    'liquidationValue': balance.get('liquidationValue'),
                    'cashBalance': balance.get('cashBalance'),
                    'availableFunds': balance.get('availableFunds'),
                    },
        }}
        # Handle and parse response
        if data is not None and "PortfolioResponse" in data and "AccountPortfolio" in data["PortfolioResponse"]:
            for acctPortfolio in data["PortfolioResponse"]["AccountPortfolio"]:
                if acctPortfolio is not None and "Position" in acctPortfolio:
                    for position in acctPortfolio["Position"]:
                        assetType = position['Product']["securityType"].replace('EQ', 'stock').replace('OPTN', 'OPTION')
                        pos = {
                            "longQuantity" : position['quantity'] if position['positionType'] == 'LONG' else 0,
                            "symbol": position['Product']["symbol"],
                            "marketValue": position['marketValue'],
                            "assetType": assetType,
                            "averagePrice": position['costPerShare'],
                            "currentDayProfitLoss": position['totalGainPct'],
                            "currentDayProfitLossPercentage": position['totalGainPct'],
                            'instrument': {'symbol': position['Product']["symbol"],
                                           'assetType': assetType,
                                           }
                        }
                        acc_inf['securitiesAccount']['positions'].append(pos)
        else:
            print("No portfolio")
            
        # get orders and add them to acc_inf
        orders = self.get_orders()
        orders_inf =[]
        for order in orders:
            order_status = order['OrderDetail'][0]['status'].upper()
            if order_status in ['CANCELLED', 'REJECTED', 'EXPIRED']:
                continue
            orders_inf.append(self.format_order(order))
        acc_inf['securitiesAccount']['orderStrategies'] = orders_inf
        return acc_inf

    def get_positions_orders(self):
        acc_inf = self.get_account_info()

        df_pos = pd.DataFrame(columns=["symbol", "asset", "type", "Qty", "Avg Price", "PnL", "PnL %"])

        for pos in acc_inf['securitiesAccount']['positions']:
            long = True if pos["longQuantity"]>0 else False

            pos_inf = {
                "symbol":pos["instrument"]["symbol"],
                "asset":pos["instrument"]["assetType"],
                "type": "long" if  long else "short",
                "Avg Price": pos['averagePrice'],
                "PnL": pos["currentDayProfitLoss"],
                }
            pos_inf["Qty"] = int(pos[f"{pos_inf['type']}Quantity"])
            pos_inf["PnL %"] = pos_inf["PnL"]/(pos_inf["Avg Price"]*pos_inf["Qty"])
            df_pos =pd.concat([df_pos, pd.DataFrame.from_records(pos_inf, index=[0])], ignore_index=True)

        df_ordr = pd.DataFrame(columns=["symbol", "asset", "type", "Qty",
                                        "Price", "action"])
        return df_pos, df_ordr

    def format_option(self, opt_ticker:str)->str:
        """From ticker_monthdayyear[callput]strike to ticker:year:month:day:optionType:strikePrice"""

        exp = r"(\w+)_(\d{2})(\d{2})(\d{2})([CP])([\d.]+)"        
        match = re.search(exp, opt_ticker, re.IGNORECASE)
        if match:
            symbol, mnt, day, yer, type, strike = match.groups()
            if type.lower() == 'c':
                type = 'Call'
            converted_code = f"{symbol}:20{yer}:{mnt}:{day}:{type}:{strike}"
            return converted_code
        else:
            print('No format_option match for', opt_ticker)

    def get_quotes(self, symbol:list):
        """
        Calls quotes API to provide quote details for equities, options, and mutual funds
        """
        # reformat option tickers

        symbol = [self.format_option(i) if "_" in i else i for i in symbol ]
        
        resp = {}
        for ix in range(0,len(symbol),25):  # lim is 25, loop over 25 symbols at a time
            symbol_l = symbol[ix:ix+25]
            data = self.market_session.get_quote(symbol_l,resp_format='json')

            if data is not None and "QuoteResponse" in data:
                if data["QuoteResponse"].get("Messages"):
                    for message in data["QuoteResponse"]["Messages"]['Message']:
                        ticker = message["description"].split(" ")[0]
                        if ":" in ticker:
                            parts = ticker.split(":")
                            ticker = f"{parts[0]}_{parts[2]}{parts[3]}{parts[1][2:]}{parts[4]}{parts[5].replace('.0','')}"
                        resp[ticker] = {'description': 'Symbol not found'}
                if "QuoteData" in data["QuoteResponse"]:
                    for quote in data["QuoteResponse"]["QuoteData"]:
                        if quote is None:
                            resp[ticker] = {'description': 'Symbol not found'}
                            continue 
                        if  quote.get("Product").get('securityType') == "OPTN":
                            # to common format ticker_monthdayyearstrike[callput]
                            prod = quote['Product']
                            opty = prod['callPut'][0].upper().replace('CALL','C').replace('PUT','P')
                            ticker = f"{prod['symbol']}_{prod['expiryMonth']:02d}{prod['expiryDay']:02d}{str(prod['expiryYear'])[2:]}{opty}{str(prod['strikePrice']).replace('.0','')}"
                        else:
                            ticker =  quote.get("Product").get('symbol')        
                        resp[ticker] = {
                            'symbol' : ticker,
                            'description': quote.get("All").get("symbolDescription"),
                            'askPrice': quote.get("All").get("ask"),  
                            'bidPrice': quote.get("All").get("bid"),    
                            'quoteTimeInLong': quote.get("dateTimeUTC")*1000,
                            "status": quote['quoteStatus']
                            }
                        if resp[ticker]['status'].upper() == 'DELAYED':
                            print("\033[91mWARNING: QUOTES ARE DELAYED by 15 min, setup realtime quotes in etrade.com. Info in github README.md \033[0m")
        return resp

    def get_order_info(self, order_id): 
        """ Get order info from order_id, mimicks the order_info from TDA API"""
        orders = self.order_session.list_orders(self.accountIdKey, resp_format='json')
        
        for order in orders['OrdersResponse']['Order']:
            if order['orderId'] == order_id:
                order_status = order['OrderDetail'][0]['status'].upper()
                order_info = self.format_order(order)         
                return order_status, order_info
        return None, None

    def format_order(self, order:dict):
        """ output format for order_response.Order, mimicks the order_info from TDA API"""
        stopPrice= order['OrderDetail'][0]['Instrument'][0].get('stopPrice')
        timestamp = int(order['OrderDetail'][0]['placedTime'])/1000
        enteredTime = datetime.fromtimestamp(timestamp).strftime("%Y-%m-%dT%H:%M:%S+00")
        order_info = {
            'status': order['OrderDetail'][0]['status'].upper(),
            'quantity': order['OrderDetail'][0]['Instrument'][0]['orderedQuantity'],
            'filledQuantity': order['OrderDetail'][0]['Instrument'][0]['filledQuantity'],
            'price':order['OrderDetail'][0]['Instrument'][0].get('averageExecutionPrice'),
            'orderStrategyType': 'SINGLE',
            "order_id" : order['orderId'],
            "orderId": order['orderId'],
            "stopPrice": stopPrice if stopPrice else None,
            'orderType':  order['OrderDetail'][0]['priceType'],
            'enteredTime': enteredTime,
            'orderLegCollection':[{
                'instrument':{'symbol':order['OrderDetail'][0]['Instrument'][0]['Product']['symbol']},
                'instruction': order['OrderDetail'][0]['Instrument'][0]['orderAction']
            }]             
        }    
        return order_info

    def send_order(self, new_order:dict):        
        order_response =  self.order_session.place_equity_order(
            resp_format="xml",
            accountId = self.accountId,
            accountIdKey = self.accountIdKey,
            **new_order)
        
        order_id = int(order_response['PlaceOrderResponse']['OrderIds']['orderId'])
        _, ord_inf = self.get_order_info(order_id)
        
        order_response['quantity'] =  int(order_response['PlaceOrderResponse']['Order']['Instrument']['quantity']),
        order_response.update(ord_inf) 
        return order_response, order_id
    
    def cancel_order(self, order_id:int):        
        return self.order_session.cancel_order(self.accountIdKey,order_id, resp_format='xml')

    def make_BTO_lim_order(self, Symbol:str, uQty:int, price:float, **kwarg):
        "Buy with a limit order"
        kwargs = {}
        kwargs['symbol'] = Symbol
        kwargs['orderAction'] = "BUY"
        if len(Symbol.split("_"))>1:
            Symbol = self.format_option(Symbol)            
            symbol, year, month, day, optype, strike = Symbol.split(":")
            kwargs['symbol'] = symbol
            kwargs['expiryDate'] = f"{year}-{month}-{day}"
            kwargs['strikePrice'] = float(strike)
            kwargs['callPut'] = "CALL" if optype.lower() == 'c' else 'PUT'
            kwargs["securityType"] = "OPTN"
            kwargs['orderAction'] = 'BUY_OPEN'        
        kwargs['clientOrderId'] = str(random.randint(1000000000, 9999999999))
        kwargs['priceType'] = 'LIMIT'
        kwargs['limitPrice'] = price    
        kwargs['allOrNone'] = False
        kwargs['quantity'] = uQty       
        kwargs['orderTerm'] = "GOOD_FOR_DAY"
        kwargs['marketSession'] = 'REGULAR'
        return kwargs

    def make_Lim_SL_order(self, Symbol:str, uQty:int,  PT:float, SL:float,  **kwarg):
        """Sell with a limit order and a stop loss order"""
        kwargs = {}
        kwargs['symbol'] = Symbol
        kwargs['orderAction'] = "SELL"
        if len(Symbol.split("_"))>1:
            Symbol = self.format_option(Symbol)            
            symbol, year, month, day, optype, strike = Symbol.split(":")
            kwargs['symbol'] = symbol
            kwargs['expiryDate'] = f"{year}-{month}-{day}"
            kwargs['strikePrice'] = float(strike)
            kwargs['callPut'] = "CALL" if optype.lower() == 'c' else 'PUT'
            kwargs["securityType"] = "OPTN"
            kwargs['orderAction'] = 'SELL_CLOSE'
        kwargs['clientOrderId'] = str(random.randint(1000000000, 9999999999))
        kwargs['priceType'] = 'STOP_LIMIT'
        kwargs['limitPrice'] = PT
        kwargs['stopPrice'] = SL
        kwargs['allOrNone'] = False
        kwargs['quantity'] = uQty       
        kwargs['orderTerm'] = "GOOD_UNTIL_CANCEL"
        kwargs['marketSession'] = 'REGULAR'
        return kwargs

    def make_STC_lim(self, Symbol:str, uQty:int, price:float, strike=None, **kwarg):
        """Sell with a limit order and a stop loss order"""
        kwargs = {}
        kwargs['symbol'] = Symbol
        kwargs['orderAction'] = "SELL"
        if len(Symbol.split("_"))>1:
            Symbol = self.format_option(Symbol)            
            symbol, year, month, day, optype, strike = Symbol.split(":")
            kwargs['symbol'] = symbol
            kwargs['expiryDate'] = f"{year}-{month}-{day}"
            kwargs['strikePrice'] = float(strike)
            kwargs['callPut'] = "CALL" if optype.lower() == 'c' else 'PUT'
            kwargs["securityType"] = "OPTN"
            kwargs['orderAction'] = 'SELL_CLOSE'
        kwargs['clientOrderId'] = str(random.randint(1000000000, 9999999999))
        kwargs['priceType'] = 'LIMIT'
        kwargs['limitPrice'] = price
        kwargs['allOrNone'] = False
        kwargs['quantity'] = uQty       
        kwargs['orderTerm'] = "GOOD_UNTIL_CANCEL"
        kwargs['marketSession'] = 'REGULAR'
        return kwargs

    def make_STC_SL(self, Symbol:str, uQty:int, SL:float, **kwarg):
        """Sell with a stop loss order"""
        kwargs = {}
        kwargs['symbol'] = Symbol
        kwargs['orderAction'] = "SELL"
        if len(Symbol.split("_"))>1:
            Symbol = self.format_option(Symbol)            
            symbol, year, month, day, optype, strike = Symbol.split(":")
            kwargs['symbol'] = symbol
            kwargs['expiryDate'] = f"{year}-{month}-{day}"
            kwargs['strikePrice'] = float(strike)
            kwargs['callPut'] = "CALL" if optype.lower() == 'c' else 'PUT'
            kwargs["securityType"] = "OPTN"
            kwargs['orderAction'] = 'SELL_CLOSE'
        kwargs['clientOrderId'] = str(random.randint(1000000000, 9999999999))
        kwargs['priceType'] = 'STOP'
        kwargs['stopPrice'] = SL
        kwargs['allOrNone'] = False
        kwargs['quantity'] = uQty       
        kwargs['orderTerm'] = "GOOD_UNTIL_CANCEL"
        kwargs['marketSession'] = 'REGULAR'

    def make_STC_SL_trailstop(self, Symbol:str, uQty:int,  trail_stop_percent:float, **kwarg):
        kwargs = {}
        kwargs['symbol'] = Symbol
        kwargs['orderAction'] = "SELL"
        if len(Symbol.split("_"))>1:
            Symbol = self.format_option(Symbol)            
            symbol, year, month, day, optype, strike = Symbol.split(":")
            kwargs['symbol'] = symbol
            kwargs['expiryDate'] = f"{year}-{month}-{day}"
            kwargs['strikePrice'] = float(strike)
            kwargs['callPut'] = "CALL" if optype.lower() == 'c' else 'PUT'
            kwargs["securityType"] = "OPTN"
            kwargs['orderAction'] = 'SELL_CLOSE'       
        kwargs['clientOrderId'] = str(random.randint(1000000000, 9999999999))
        kwargs['priceType'] = 'TRAILING_STOP_PRCT'
        kwargs['stopPrice'] = trail_stop_percent
        kwargs['allOrNone'] = False
        kwargs['quantity'] = uQty       
        kwargs['orderTerm'] = "GOOD_UNTIL_CANCEL"
        kwargs['marketSession'] = 'REGULAR'
        return kwargs

    def get_orders(self):
        orders = self.order_session.list_orders(self.accountIdKey, resp_format='json')
        orders = orders['OrdersResponse']['Order']
        """   {'orderId': 3,
    'details': 'https://api.etrade.com/v1/accounts/kx-cz1Rmn1w_MMgg2K7wCA/orders/3.json',
    'orderType': 'EQ',
    'OrderDetail': [{'placedTime': 1684804106634,
      'executedTime': 1178021968,
      'orderValue': 1.0098,
      'status': 'CANCELLED',
      'orderTerm': 'GOOD_UNTIL_CANCEL',
      'priceType': 'TRAILING_STOP_PRCT',
      'priceValue': 'Tsp * 1.01',
      'limitPrice': 0,
      'stopPrice': 0,
      'offsetType': 'TRAILING_STOP_PRCT',
      'offsetValue': 40,
      'marketSession': 'REGULAR',
      'bracketedLimitPrice': 0,
      'initialStopPrice': 1.01,
      'trailPrice': 0,
      'triggerPrice': 0,
      'allOrNone': False,
      'netPrice': 0,
      'netBid': 0,
      'netAsk': 0,
      'gcd': 0,
      'ratio': '',
      'Instrument': [{'symbolDescription': 'VERB TECHNOLOGY CO INC COM NEW',
        'orderAction': 'SELL',
        'quantityType': 'QUANTITY',
        'orderedQuantity': 1,
        'filledQuantity': 0.0,
        'estimatedCommission': 0.0001,
        'estimatedFees': 0.0,
        'Product': {'symbol': 'VERB', 'securityType': 'EQ'}}]}]},
   {'orderId': 2,
    'details': 'https://api.etrade.com/v1/accounts/kx-cz1Rmn1w_MMgg2K7wCA/orders/2.json',
    'orderType': 'EQ',
    'OrderDetail': [{'placedTime': 1684414597753,
      'executedTime': 1684416602677,
      'orderValue': 1.75,
      'status': 'EXECUTED',
      'orderTerm': 'GOOD_FOR_DAY',
      'priceType': 'MARKET',
      'limitPrice': 0,
      'stopPrice': 0,
      'marketSession': 'REGULAR',
      'allOrNone': False,
      'netPrice': 0,
      'netBid': 0,
      'netAsk': 0,
      'gcd': 0,
      'ratio': '',
      'Instrument': [{'symbolDescription': 'VERB TECHNOLOGY CO INC COM NEW',
        'orderAction': 'BUY',
        'quantityType': 'QUANTITY',
        'orderedQuantity': 1,
        'filledQuantity': 1.0,
        'averageExecutionPrice': 1.76,
        'estimatedCommission': 0.0,
        'estimatedFees': 0.0,
        'Product': {'symbol': 'VERB', 'securityType': 'EQ'}}]}]},"""
        return orders



if 0:
    rt = eTrade()
    rt.get_session()
    rt.get_account_info()
    rt.get_quotes(["AAPL", 'NIO:2023:08:18:P:4'])

    order = rt.make_STC_lim('NIO_081823P4', 1, 0.07)
    order_response, order_id = rt.send_order(order)

    order = rt.make_BTO_lim_order('VERB', 1, 1.7)
    order_response, order_id = rt.send_order(order)
    res = rt.cancel_order(order_id)

    order = rt.make_Lim_SL_order('VERB', 1,  1.8, 1.5)
    order_response, order_id = rt.send_order(order)
    res = rt.cancel_order(order_id)
    
    order = rt.make_STC_SL_trailstop('VERB', 1, 40)
    order_response, order_id = rt.send_order(order)
    res = rt.cancel_order(order_id)